/*! Built with http://stenciljs.com */
const { h } = window.xmlviewercomponent;

class Parser {
    static Parse(xmlstr) {
        if (typeof DOMParser !== 'undefined') {
            // const DOMParser = new Components.Constructor("@mozilla.org/xmlextras/domparser;1", "nsIDOMParser");
            // var parser = new DOMParser();
            // parser.init();
            // var doc = parser.parseFromString(xmlstr, "application/xml");
            let domDoc = (new DOMParser()).parseFromString(xmlstr, 'application/xml');
            // let's check for errors
            if (domDoc.documentElement.nodeName == "parsererror") {
                throw new Error('XML parsing error');
            }
            return domDoc;
        }
        return null;
    }
}

class XmlViewerComponent {
    componentDidLoad() {
        console.log('The component has been rendered');
    }
    prepareXml() {
        // let regex = /&lt;([\w\d]+)/ig;
        // let regexClose = /&lt;\/([\w\d]+)&gt;/ig;
        // let regexLt = /(\<)/ig;
        // let regexRt = /(\>)/ig;
        // let regexAttr = /([\w\d\-\_]+)\=/ig;
        // replace
        // let xml = null;
        let xdoc = Parser.Parse(this.xml);
        // try{
        //   xml = this.xml
        //   .replace(regexLt, "&lt;")
        //   .replace(regexRt, "&gt;")
        //   // replace attributes
        //   .replace(regexAttr, "<span class='attr'>$1</span>=")
        //   // replace element start
        //   .replace(regex, `&lt;<span class='element'>$1</span>`)
        //   // replace element end
        //   .replace(regexClose, "&lt;/<span class='element'>$1</span>&gt;")
        //   // replace tabs and spaces with nonbreakable spaces
        //   .replace(/^[\s\t]/ig, '&nbsp;');
        // }
        // catch(e){
        //   console.log(e);
        // }
        // in case parsing fails
        // return xml || this.xml;
        return xdoc;
    }
    renderAttribute(attribute) {
        return (h("span", null,
            "\u00A0",
            h("span", { class: "attr" }, attribute.name),
            h("span", null,
                "=\"",
                attribute.value,
                "\"")));
    }
    renderNodeValue(nodeValue) {
        if (!nodeValue) {
            return null;
        }
        if (nodeValue.length > 150) {
            return (h("ul", null,
                h("li", null, nodeValue)));
        }
        else {
            return (h("span", null, nodeValue));
        }
    }
    // rendering node. This function calls to itself in recursion way in case of child nodes
    renderNode(node) {
        if (!node) {
            return null;
        }
        let children = [...node.children];
        let attributes = [...node.attributes];
        let nodeValue = node.firstChild ? node.firstChild.nodeValue : null;
        nodeValue = nodeValue ? nodeValue.trim() : null;
        return (h("ul", null,
            h("li", null,
                h("span", { class: "element" }, "<"),
                h("span", { class: "element" }, node.nodeName),
                attributes.map((a) => this.renderAttribute(a)),
                h("span", { class: "element" }, ">"),
                this.renderNodeValue(nodeValue),
                children.map((c) => this.renderNode(c)),
                h("span", { class: "element" }, "<"),
                h("span", { class: "element" }, node.nodeName),
                h("span", { class: "element" }, "\u00A0/>"))));
    }
    render() {
        console.log('rendered');
        if (this.xml === null || this.xml === undefined) {
            return null;
        }
        if (this.xml.length === 0) {
            return null;
        }
        let xdoc = this.prepareXml();
        if (!xdoc || !xdoc.documentElement.childNodes) {
            return null;
        }
        //let arr = [1,2,3,4,5];
        return (
        // <code innerHTML={xml}></code>
        h("code", null, this.renderNode(xdoc.documentElement)));
    }
    static get is() { return "xml-viewer-component"; }
    static get encapsulation() { return "shadow"; }
    static get properties() { return { "xml": { "type": String, "attr": "xml" } }; }
    static get style() { return "code[data-xml-viewer-component]{\n    margin:.1rem .25rem 1rem .25rem;\n    padding:0;\n    color:#767676;\n    font-family: monospace, monospace;\n\n    text-shadow: 0 1px #fff;\n    direction: ltr;\n    text-align: left;\n    word-spacing: normal;\n    word-break: normal;\n    word-wrap: normal;\n    line-height: 1.2;\n    -moz-tab-size: 2; -o-tab-size: 2; tab-size: 2;\n    -webkit-hyphens: none; -moz-hyphens: none; -ms-hyphens: none; hyphens: none;\n}\n\ncode[data-xml-viewer-component]   ul[data-xml-viewer-component]{\n    list-style-type: none;\n    margin:0;\n    padding-left:1rem;\n}\n\ncode[data-xml-viewer-component]   span.element[data-xml-viewer-component]{\n    font-weight: bold;\n    color: darkred;\n}\n\ncode[data-xml-viewer-component]   span.attr[data-xml-viewer-component]{\n    font-weight: bold;\n    color: #535353;\n}\n\ncode[data-xml-viewer-component]   span.comment[data-xml-viewer-component]{\n    color: darkgreen;\n}"; }
}

export { XmlViewerComponent };
